Structs
- Dep
- Static dependency container (Heterogeneous List).
- Here
- Type-level index indicating the item is found Here (at the head).
- There
- Type-level index indicating the item is found There (in the tail).
Traits
- Has
- Trait to extract a dependency
Tusing a specificIndex.