Dep

Struct Dep 

pub struct Dep<Head, Tail>(pub Head, pub Tail);
Static dependency container (Heterogeneous List).

This structure allows storing mixed types of dependencies and accessing them in a zero-cost manner (offsets calculated at compile time).

Tuple Fields

0: Head1: Tail

Implementations

Source

impl Dep<(), ()>

pub fn new() -> Self

Source

impl<Head, Tail> Dep<Head, Tail>

pub fn add<T>(self, item: T) -> Dep<T, Self>

Prepend a new dependency to the list.

pub fn get<T, I>(&self) -> &T
where Self: Has<T, I>,

Access a dependency by type.

The index I is automatically inferred by the compiler. This method shadows the trait method to provide better type inference.

pub fn get_mut<T, I>(&mut self) -> &mut T
where Self: Has<T, I>,

access a mutable dependency by type.