Module ext
pipe_it
0.1.0
Module ext
Module Items
Structs
Traits
In crate pipe_
it
pipe_it
Module
ext
Structs

Cache
Connect
Extend
Pipe
Wrapper struct to adapt a Handler into a Pipeline.
Pullback
Repeat
Traits

Handler
Ext
Extension trait for any type that implements the Handler trait. Provides a way to convert a Handler into a Pipeline and combinators.