Context

Struct Context 

pub struct Context<Input>
where Input: ?Sized,
{ /* private fields */ }
The execution context for a pipeline, carrying both shared dependencies and the specific input.

Implementations

Source

impl<I> Context<I>

pub fn new(input: I, shared: Shared) -> Self

Creates a new context with the provided input and shared dependency collection.

pub fn empty(input: I) -> Self

pub async fn invoke<H, Args, O>(self, handler: H) -> Context<O>
where H: Handler<I, O, Args>, I: Clone + Send + Sync + 'static, O: Send + Sync + 'static, Args: Send + Sync + 'static,

Example
use pipe_it::{Context, Input, Res, ResMut, Shared};

#[derive(Debug, Clone)]
struct Counter {
    c: i32,
}

#[tokio::main]
async fn main() {
    let ctx = Context::new(3, Shared::new().insert(Counter { c: 1 }));
    ctx.invoke(async |x: Input<i32>, mut counter: ResMut<Counter>| {
        counter.c += 1;
        *x + 1
    })
    .await
    .invoke(async |x: Input<i32>, counter: Res<Counter>| *x + counter.c)
    .await
    .invoke(async |x: Input<i32>| assert_eq!(*x, 6))
    .await;
}