Input

Struct Input 

pub struct Input<T>(pub Arc<T>);
A wrapper for the primary input data of the pipeline.

Tuple Fields

0: Arc<T>

Implementations

Source

impl<T> Input<T>

pub fn try_unwrap(self) -> Result<T, Arc<T>>

Returns the inner value, if the Arc has exactly one strong reference. Otherwise, an Err is returned with the same Arc that was passed in. // Optimized: consume context to steal Arc without cloning if possible